package com.dowerx.quack.service;

import android.app.Activity;
import android.content.Context;
import android.content.Intent;
import android.os.AsyncTask;
import android.util.Log;
import android.widget.Toast;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.loader.content.AsyncTaskLoader;

import com.dowerx.quack.R;
import com.dowerx.quack.model.Post;
import com.dowerx.quack.model.User;
import com.google.android.gms.tasks.Task;
import com.google.android.gms.tasks.Tasks;
import com.google.firebase.auth.FirebaseAuth;
import com.google.firebase.firestore.CollectionReference;
import com.google.firebase.firestore.DocumentSnapshot;
import com.google.firebase.firestore.Filter;
import com.google.firebase.firestore.FirebaseFirestore;
import com.google.firebase.firestore.Query;
import com.google.firebase.firestore.QueryDocumentSnapshot;
import com.google.firebase.firestore.QuerySnapshot;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.concurrent.ExecutionException;
import java.util.concurrent.atomic.AtomicBoolean;

public class PostService {
    public static class GetFeed extends AsyncTaskLoader<List<Post>> {

        public GetFeed(@NonNull Context context) {
            super(context);
        }

        @Nullable
        @Override
        public List<Post> loadInBackground() {
            List<Post> posts = new LinkedList<>();

            FirebaseFirestore db = FirebaseFirestore.getInstance();
            try {
                for (QueryDocumentSnapshot post : Tasks.await(db.collection("/post").limit(10).get())) {
                    DocumentSnapshot user = Tasks.await(post.getDocumentReference("user").get());
                    String username = user.get("username", String.class);
                    String email = user.get("email", String.class);
                    String id = user.get("id", String.class);
                    posts.add(new Post(
                            new User(username, email, id),
                            post.get("content", String.class),
                            post.get("image", String.class),
                            0,
                            false,
                            post.getId()
                    ));
                }
            } catch (ExecutionException | InterruptedException e) {
                throw new RuntimeException(e);
            }

            return posts;
        }

        @Override
        protected void onStartLoading() {
            forceLoad();
        }
    }

    public static class GetProfileFeed extends AsyncTaskLoader<List<Post>> {

        private final String username;

        public GetProfileFeed(@NonNull Context context, String username) {
            super(context);
            this.username = username;
        }

        @Nullable
        @Override
        public List<Post> loadInBackground() {
            List<Post> posts = new LinkedList<>();

            FirebaseFirestore db = FirebaseFirestore.getInstance();
            try {
                for (QueryDocumentSnapshot post : Tasks.await(db.collection("/post")
                        .where(Filter.equalTo("user", db.document("/user/"+username))).get())) {
                    DocumentSnapshot user = Tasks.await(post.getDocumentReference("user").get());
                    String username = user.get("username", String.class);
                    String email = user.get("email", String.class);
                    String id = user.get("id", String.class);
                    posts.add(new Post(
                            new User(username, email, id),
                            post.get("content", String.class),
                            post.get("image", String.class),
                            0,
                            false,
                            post.getId()
                    ));
                }
            } catch (ExecutionException | InterruptedException e) {
                throw new RuntimeException(e);
            }

            return posts;
        }

        @Override
        protected void onStartLoading() {
            forceLoad();
        }
    }

    public static class NewPost extends AsyncTask<String, Void, Boolean> {

        Activity activity;

        public NewPost(Activity activity) {
            this.activity = activity;
        }

        @Override
        protected Boolean doInBackground(String... strings) {
            FirebaseFirestore db = FirebaseFirestore.getInstance();
            FirebaseAuth fba = FirebaseAuth.getInstance();

            Log.d(this.getClass().getName(), "before adding post");

            try {
                DocumentSnapshot user = Tasks.await(db.collection("user").where(Filter.equalTo("id", fba.getUid())).get()).getDocuments().get(0);
                HashMap<String, Object> post = new HashMap<>();
                post.put("content", strings[0]);
                post.put("image", "");
                post.put("user", user.getReference());
                Tasks.await(db.collection("post").add(post));

                Log.d(this.getClass().getName(), "after adding post");

            } catch (ExecutionException | InterruptedException e) {
                return false;
            }
            return true;
        }

        @Override
        protected void onPostExecute(Boolean result) {
            if (result) {
                activity.findViewById(R.id.nav_profile).performClick();
            } else {
                Toast toast = new Toast(activity);
                toast.setDuration(Toast.LENGTH_LONG);
                toast.setText("failed to send post");
                toast.show();
            }
        }
    }
}
